

#ifndef _psdev_h_ /* Fri Mar 14 15:46:50 1997 */
#define _psdev_h_



/*
 *
 *          Copyright (C) 1997, M. A. Sridhar
 *  
 *
 *     This software is Copyright M. A. Sridhar, 1997. You are free
 *     to copy, modify or distribute this software  as you see fit,
 *     and to use  it  for  any  purpose, provided   this copyright
 *     notice and the following   disclaimer are included  with all
 *     copies.
 *
 *                        DISCLAIMER
 *
 *     The author makes no warranties, either expressed or implied,
 *     with respect  to  this  software, its  quality, performance,
 *     merchantability, or fitness for any particular purpose. This
 *     software is distributed  AS IS.  The  user of this  software
 *     assumes all risks  as to its quality  and performance. In no
 *     event shall the author be liable for any direct, indirect or
 *     consequential damages, even if the  author has been  advised
 *     as to the possibility of such damages.
 *
 */


#include "ui/dwgsurf.h"


// This is a PostScript page accessible through a DrawingSurface
// protocol. It expects an ostream as one of its constructor
// parameters. Typical use of this class is as follows:
//
//        ofstream psfile ("file.ps");
//        UI_PostscriptDevice dev (psfile);
//
//        // .... Drawing commands here ...
//
//        dev.Close ();
//
// After the Close method is called, the PostscriptDevice will have
// appended to the ostream the PostScript program that would draw the
// graphics spcified by the drawing commands.
//
// This class treats a pixel as synonymous with a point (1 inch = 72.24
// points).

class YACL_UI UI_PostscriptDevice: public UI_DrawingSurface {

public:

    UI_PostscriptDevice (ostream& stream, double pageWidthInches = 8.5,
			 double pageHeightInches = 11.0,
                         const char* title = 0, const char* creatorName = 0);

    ~UI_PostscriptDevice ();


    // 
    // --------------------------- Query ---------------------------------
    //     

    virtual double HorzPixelsPerMM () const;
    // Return the number of pixels per millimeter in the horizontal
    // direction.

    virtual double VertPixelsPerMM () const;
    // Return the number of pixels per millimeter in the vertical direction.

    virtual UI_Rectangle DrawingArea() const;
    // Returns the rectangle enclosing the drawing surface, in pixels. The
    // origin is (0, 0).

    virtual UI_Rectangle DrawingAreaInMM () const;
    
    // 
    // -------------------------- Draw  operations ------------------------
    // 


    virtual bool DrawLine (const UI_Point& p, const UI_Point& q);
    // Draw a line from {\tt p} to {\tt q}, using our pen.

    virtual void DrawRectangle (const UI_Rectangle& rect,
                                ushort opt = UID_Outline); 
    // Draw a rectangle using the current pen. The second parameter
    // specifies the outline and fill options.
                
    virtual void DrawEllipse (const UI_Rectangle& rect,
                              ushort opt = UID_Outline); 
    // Draw an ellipse with given bounding rectangle, using the current pen.
    // The second parameter specifies the outline and fill options.
    // (If the "rectangle" happens to be a square, then a circle is drawn.)
                
    virtual void DrawArc (const UI_Arc &arc);
    // Draw an arc with the using the parameters given to UI_Arc.

    virtual void DrawChord (const UI_Chord& chord,
                            ushort opt = UID_Outline); 
    // Draw a chord using the current pen. The second parameter
    // specifies the outline and fill options.

    virtual void DrawPieWedge (const UI_PieWedge& pie,
                               ushort opt = UID_Outline); 
    // Draw a pie wedge using the current pen. The second parameter
    // specifies the outline and fill options.
        
    virtual void DrawPolygon (UI_Point pointArray[], short numPoints,
                              ushort opt = UID_Outline);
    // Draw a closed polygon by connecting the points in the array with
    // line segments, in the order of occurrence of the points.  The second
    // parameter specifies the outline and fill options.
    // The first point in the array is taken as the starting point.

    virtual void DrawPolyLine (UI_Point pointArray[], short numPoints);
    // Draw a figure by connecting the points in the array with
    // line segments, in the order of occurrence of the points. 
    // The first point in the array is taken as the starting point.

    virtual void DrawPoint (const UI_Point& p);
    // Draw a point at position {\tt p} with the brush's color.

    // virtual bool DrawBitmap (const UI_Bitmap& b, const UI_Point& topLeft);
    // Draw the given bitmap on this surface.

    // 
    // ------------------------ Text operations -------------------------
    // 

    virtual void WriteString (const char* str, const UI_Rectangle& box,
                              UI_TextStyle style = UIText_Left);
    // Write the given text string within the given box, in the given
    // style. This method is guaranteed to handle a single line, not
    // necessarily multiple lines. It draws the text within the rectangle,
    // clipping the text if necessary.

    
    virtual short TextWidth (const char* s) const {return 0;};
    // Return the width (in pixels) of the text string s if it were drawn
    // on this DrawingSurface using its current font.
    //
    // Warning: the current implementation of this method returns 0
    // unconditionally. This is because we don't have a good way of
    // determining this width on a Postscript device.
    

    // ------------------------ Saving and restoring ---------------------
    virtual bool SaveState();
    // Save the current state of the DrawingSurface (i.e., its drawing
    // attributes, pen, brush, font etc.) on an internal stack. A subsequent
    // call to {\tt RestoreState} restores the most recently saved state.
    //
    // The method returns TRUE on success, FALSE on failure.

    virtual bool RestoreState ();
    // Restore the state most recently saved via the {\tt SaveState} method.
    // The return value is TRUE on success, FALSE on failure (e.g., if the
    // state stack was already empty).

    // ------------------------ Page operations --------------------------
    bool StartNewPage ();
    // Start a new page.

    // ------------------------ Device-specific extensions ---------------
    void AppendCode (const char* psString);
    // Write the specified string literally into the Postscript
    // stream. This method can be used, for example, as a means for the
    // application program to write Postscript code directly into the stream.

    ostream& PostscriptStream () const {return _stream;};
    // Return the stream that this PostscriptDevice is writing to.
    
    
    // ------------------------ Basic methods ----------------------------
    
    const char* ClassName () const {return "UI_PostscriptDevice";};

    // -------------------- End public protocol -------------------------


protected:
    double          _widthPts;
    double          _heightPts;
    long            _pageNo;
    ostream&        _stream;
    

    virtual void    _DoOutput (const char*); // All output happens through
                                             // this method.

    virtual bool    _PenChanged ();

    virtual bool    _FontChanged ();

    void            _BeginFigure (const char* n = 0);
    
    void            _EndFigure (long opts);
    
};


#endif /* _psdev_h_ */
