




/*
 *
 *          Copyright (C) 1995, M. A. Sridhar
 *  
 *
 *     This software is Copyright M. A. Sridhar, 1995. You are free
 *     to copy, modify or distribute this software  as you see fit,
 *     and to use  it  for  any  purpose, provided   this copyright
 *     notice and the following   disclaimer are included  with all
 *     copies.
 *
 *                        DISCLAIMER
 *
 *     The author makes no warranties, either expressed or implied,
 *     with respect  to  this  software, its  quality, performance,
 *     merchantability, or fitness for any particular purpose. This
 *     software is distributed  AS IS.  The  user of this  software
 *     assumes all risks  as to its quality  and performance. In no
 *     event shall the author be liable for any direct, indirect or
 *     consequential damages, even if the  author has been  advised
 *     as to the possibility of such damages.
 *
 */



#if defined(__GNUC__)
#pragma implementation
#endif


#include "ui/scrolbar.h"
#include "ui/cntroler.h"
#include "ui/interval.h"

#if defined(__MS_WINDOWS__) || defined(__MS_WIN32__)
#    include <windows.h>

#elif defined(__X_MOTIF__)
#    if defined(__GNUC__) && __GNUC_MINOR__ >= 7
#        include <string.h>  // Without this, the X includes barf
#    endif
#    include <Xm/ScrollBar.h> 
#    include <iostream.h> // DEBUG
#elif defined(__X_YACL__)
#    include <iostream.h> // DEBUG
#    include <stdlib.h>
#endif

#if defined(__GNUC__) && __GNUC_MINOR__ >= 6
template class CL_Binding0<UI_ScrollBar>;
#elif defined(_MSC_VER)
template CL_Binding0<UI_ScrollBar>;
#endif


/*------------------ ScrollBar Object ---------------------------*/

typedef CL_Binding0<UI_ScrollBar> Bind;

UI_ScrollBar::UI_ScrollBar (UI_VisualObject* p,
                            const UI_Rectangle& shape,
                            UI_ViewID id, long bstyle, bool vert)
:  UI_SimpleVObject (p, shape, id, bstyle), _range (0, 100),
   _vertical(vert)
#if defined(__X_YACL__)
    , _gutterColor (0.62, 0.58, 0.62)
#endif
   
{ 
    _model = new CL_Interval (0,0);
    _ownModel = TRUE;
    Bind bind (this, &UI_ScrollBar::_RangeChanged);
    _range.AddDependent (bind); // Second parameter unused
    _lineAmount = 1;
    _pageAmount = 10;
    _smoothScroll = TRUE;
#if defined(__X_YACL__)
    _window        = 0;
    _visualElement = NULL;
    _sliderPos     = 0;
    _sliderSize    = 40;
    _mouseIsDown   = FALSE;
    _memDS         = NULL;
#endif
}


UI_ScrollBar::~UI_ScrollBar()
{
}



UI_WindowClass UI_ScrollBar::WindowClass () const
{
#if defined(__MS_WINDOWS__) || defined(__MS_WIN32__)
    return "scrollbar";
#elif defined(__OS2__)
    return WC_SCROLLBAR;
#elif defined(__X_MOTIF__)
    return xmScrollBarWidgetClass;
#elif defined(__X_YACL__)
    return X_YACL_SCROLLBAR_CLASS;
#endif
}


void UI_ScrollBar::_PrivateInitialize()
{
    UI_SimpleVObject::_PrivateInitialize();

#if defined(__MS_WINDOWS__) || defined(__MS_WIN32__)
    if (_handle) {
        _SetRange ();
        CL_Interval& interval = *(CL_Interval*) _model;
        _SetPosition (interval.Low());
    }
#elif defined(__OS2__)
    _RangeChanged (); // Force an initialization of range
    _ModelChanged (); // Force initialization of model
#elif defined(__X_MOTIF__)
    Arg args[1];
    XtSetArg    (args[0], XmNorientation,
                 _vertical ? XmVERTICAL : XmHORIZONTAL);
    XtSetValues (_xwidget, args, 1);

    _RangeChanged (); // Force an initialization of range
    _ModelChanged (); // Force initialization of model
    XtAddCallback (_xwidget, XmNdragCallback, 
                   &UI_ScrollBar::ScrollBarCallback, (XtPointer) this);
    XtAddCallback (_xwidget, XmNincrementCallback, 
                   &UI_ScrollBar::ScrollBarCallback, (XtPointer) this);
    XtAddCallback (_xwidget, XmNpageIncrementCallback, 
                   &UI_ScrollBar::ScrollBarCallback, (XtPointer) this);
    XtAddCallback (_xwidget, XmNdecrementCallback, 
                   &UI_ScrollBar::ScrollBarCallback, (XtPointer) this);
    XtAddCallback (_xwidget, XmNpageDecrementCallback, 
                   &UI_ScrollBar::ScrollBarCallback, (XtPointer) this);
    XtAddCallback (_xwidget, XmNtoBottomCallback, 
                   &UI_ScrollBar::ScrollBarCallback, (XtPointer) this);
    XtAddCallback (_xwidget, XmNtoTopCallback, 
                   &UI_ScrollBar::ScrollBarCallback, (XtPointer) this);
    XtAddCallback (_xwidget, XmNvalueChangedCallback, 
                   &UI_ScrollBar::ScrollBarCallback, (XtPointer) this);
#endif
}


#if defined(__X_MOTIF__)
void UI_ScrollBar::ScrollBarCallback (Widget , void* client, void* call)
{
    UI_ScrollBar* bar = (UI_ScrollBar *) client;
    UI_EventType type;
    XmScrollBarCallbackStruct* cb = (XmScrollBarCallbackStruct*) call;
    switch (cb->reason) {
    case XmCR_DRAG:
        type = Event_Scroll;
        break;
        
    case XmCR_DECREMENT:
        type = Event_ScrollBackwardLine;
        break;
        
    case XmCR_INCREMENT:
        type = Event_ScrollForwardLine;
        break;
        
    case XmCR_PAGE_DECREMENT:
        type = Event_ScrollBackwardPage;
        break;
        
    case XmCR_PAGE_INCREMENT:
        type = Event_ScrollForwardPage;
        break;

    case XmCR_TO_TOP:
        type = Event_ScrollToBegin;
        break;

    case XmCR_TO_BOTTOM:
        type = Event_ScrollToEnd;
        break;

    case XmCR_VALUE_CHANGED:
        type = Event_FinishScroll;
        break;

    default:
        CL_Error::Warning ("UI_ScrollBar callback: unknown reason");
        break;
    };
    int val, slsize, inc, page_inc;
    XmScrollBarGetValues (bar->_xwidget, &val, &slsize, &inc, &page_inc);
    bar->_SetModelValue (CL_Interval (val, val+slsize-1));
    UI_Event* evt = new UI_ScrollEvent (type, bar, bar);
    evt->ScrollAmount() = val;
    _Controller->AddEvent (evt);
}

#endif // __X_MOTIF__


#if defined(__MS_WINDOWS__) || defined(__MS_WIN32__)
void UI_ScrollBar::_SetPosition (long pos)
{
    if (_handle)
        SetScrollPos (_handle, SB_CTL, pos, TRUE);
}
void UI_ScrollBar::_SetRange ()
{
    if (_handle) {
        CL_Interval& interval = *(CL_Interval*) _model;
        SetScrollRange ((HWND) _handle, SB_CTL, _range.Low(), 
                        _range.High() - interval.Length () + 1,
                        TRUE);
    }
}
#endif


#if !defined(__X_YACL__)
bool UI_ScrollBar::HandleEvent (UI_Event* e)
{
    UI_EventType type = e->Type();
    if (type < Event_Scroll || type > Event_ScrollToPosition) {
        return ProcessEvent (e);
    }

    CL_Interval& value = *(CL_Interval*) _model;
    long pos = value.Low();
    switch (type) {
    case Event_ScrollToBegin:
        pos = _range.Low();
        break;
        
    case Event_ScrollToEnd:
        pos = _range.High() - value.Length() + 1;
        break;
        
    case Event_ScrollForwardLine:
        pos = minl (_range.High(), pos + _lineAmount);
        break;
        
    case Event_ScrollBackwardLine:
        pos = maxl (_range.Low(), pos - _lineAmount);
        break;

    case Event_ScrollForwardPage:
        pos = minl (_range.High(), pos + _pageAmount);
        break;
        
    case Event_ScrollBackwardPage:
        pos = maxl (_range.Low (), pos - _pageAmount);
        break;

    case Event_ScrollToPosition:
    case Event_Scroll:
        pos = e->ScrollAmount();
        break;

    default:
        break;
    }        
#if defined(__MS_WINDOWS__) || defined(__MS_WIN32__)
    if (type != Event_FinishScroll) {
        long lo, hi, maxLo = _range.High() - value.Length() + 1;
        if (pos <= maxLo) {
            lo = pos;
            hi = pos + value.Length() - 1;
        }
        else {
            lo = maxLo;
            hi = _range.High();
        }
        _SetModelValue (CL_Interval (lo, hi));
        _SetPosition (pos);
    }
#elif defined(__OS2__)
    long lo, hi, maxLo = _range.High() - value.Length() + 1;
    if (pos <= maxLo) {
        lo = pos;
        hi = pos + value.Length() - 1;
    }
    else {
        lo = maxLo;
        hi = _range.High();
    }
    _SetModelValue (CL_Interval (lo, hi));
    if (type != Event_FinishScroll)
        WinSendMsg (_handle, SBM_SETPOS, MPFROMSHORT (pos), 0);
#endif
    if (type != Event_Scroll || _smoothScroll) {
        _clientSet.NotifyAll (*e);
    }
    return FALSE; // So the parent gets to see the event
}
#endif // Not X_YACL

CL_Interval& UI_ScrollBar::Range ()
{
    return _range;
}



#if defined(__X_MOTIF__)
static void _SetResources (Widget scroll, const CL_Interval& model,
                           const CL_Interval& range, short line, short page)
{
    if (model.Low () < range.Low() || model.High() > range.High())
        return; // Refuse the change
    Arg args[2];
    XtSetArg    (args[0] , XmNminimum, range.Low  ());
    XtSetArg    (args[1] , XmNmaximum, range.High () /* + model.Length() */);
    XtSetValues (scroll, args, 2);
    int sliderPos  = maxl (range.Low(), minl (model.Low(), range.High()));
    int sliderSize = maxl (1, minl (model.Length(), range.Length()));
    sliderSize = minl (sliderSize, range.High() - range.Low());
    XmScrollBarSetValues  (scroll, sliderPos, sliderSize,  line, page,
                          FALSE);
//     UI_ScrollBar* b = (UI_ScrollBar*) YACLApp()->Controller()[scroll];
//     if (b)
//         cout << b->InstanceName () << " ";
//     cout << "sliderPos " << sliderPos << " sliderSize "
//          << sliderSize 
//          << " model " << model << " range " << range << endl; // DEBUG
}

#endif

bool UI_ScrollBar::_ModelChanged ()
{
#if defined(__MS_WINDOWS__) || defined(__MS_WIN32__)
    _SetPosition (((CL_Interval*) _model)->Low());
#elif defined(__OS2__)
    if (_handle) {
        CL_Interval& value = *(CL_Interval*) _model;
        short range = _range.High() - _range.Low() + 1 - value.Length();
        WinSendMsg (_handle, SBM_SETSCROLLBAR, (MPARAM) value.Low(),
                    MPFROM2SHORT (0, range));
        WinSendMsg (_handle, SBM_SETTHUMBSIZE,
                    MPFROM2SHORT (value.Length(), _range.Length()), 0);
    }
#elif defined(__X_MOTIF__)
    if (_xwidget) {
        CL_Interval& value = *(CL_Interval*) _model;
        _SetResources (_xwidget, value, _range, _lineAmount, _pageAmount);
    }
#elif defined(__X_YACL__)
    _AdjustModel ();
    _ConfigureSlider();
    Invalidate ();
#endif
    return TRUE;
}

bool UI_ScrollBar::_RangeChanged ()
{
#if defined(__MS_WINDOWS__) || defined(__MS_WIN32__)
    _SetRange ();
#elif defined(__OS2__)
    if (_handle) {
        CL_Interval& value = *(CL_Interval*) _model;
        short range = _range.High() - _range.Low() - 1 - value.Length();
        WinSendMsg (_handle, SBM_SETSCROLLBAR, (MPARAM) value.Low(),
                    MPFROM2SHORT (0, range));
        WinSendMsg (_handle, SBM_SETTHUMBSIZE,
                    MPFROM2SHORT (value.Length(), _range.Length()), 0);
    }
                    
#elif defined(__X_MOTIF__)
    if (_xwidget) {
        CL_Interval& value = *(CL_Interval*) _model;
        _SetResources (_xwidget, value, _range, _lineAmount, _pageAmount);
    }
#elif defined(__X_YACL__)
    _AdjustModel ();
    _ConfigureSlider();
    Invalidate ();
#endif
    return TRUE;
}



/*---------------------VScrollBar------------------------------------*/

UI_VScrollBar::UI_VScrollBar
    (UI_VisualObject* p, const UI_Rectangle& shape,
     UI_ViewID id, long bstyle)
: UI_ScrollBar (p, shape, id, bstyle, TRUE)
{
#if defined (__MS_WINDOWS__) || defined(__MS_WIN32__)
    if (_style == 0xffffffffL)
        _style = WS_VISIBLE | SBS_VERT | WS_CHILD;
#elif defined (__OS2__)
    _style = SBS_VERT | WS_VISIBLE;
#endif
}




/*-------------------------HScrollBar----------------------------*/

UI_HScrollBar::UI_HScrollBar
    (UI_VisualObject* p, const UI_Rectangle& shape,
     UI_ViewID id, long bstyle)
: UI_ScrollBar (p, shape, id, bstyle, FALSE)
{
#if defined(__MS_WINDOWS__) || defined(__MS_WIN32__)
    if (_style == 0xffffffffL)
        _style = WS_VISIBLE | SBS_HORZ | WS_CHILD;
#elif defined (__OS2__)
    _style = SBS_HORZ | WS_VISIBLE;
#endif
}
