

#ifndef _treevmdl_h_ /* Sat Jan 25 17:20:26 1997 */
#define _treevmdl_h_




/*
 *
 *          Copyright (C) 1997, M. A. Sridhar
 *  
 *
 *     This software is Copyright M. A. Sridhar, 1997. You are free
 *     to copy, modify or distribute this software  as you see fit,
 *     and to use  it  for  any  purpose, provided   this copyright
 *     notice and the following   disclaimer are included  with all
 *     copies.
 *
 *                        DISCLAIMER
 *
 *     The author makes no warranties, either expressed or implied,
 *     with respect  to  this  software, its  quality, performance,
 *     merchantability, or fitness for any particular purpose. This
 *     software is distributed  AS IS.  The  user of this  software
 *     assumes all risks  as to its quality  and performance. In no
 *     event shall the author be liable for any direct, indirect or
 *     consequential damages, even if the  author has been  advised
 *     as to the possibility of such damages.
 *
 */


// The TreeViewModel is the object that serves as model for the
// TreeView. Conceptually, it is a  tree whose nodes contain pointers to
// instances of CL_Object. When the TreeView displays each node, it uses the
// AsString method on the content instance, and displays the returned string
// as the text of the node. Applications can therefore use instances of any
// class derived from CL_Object as node content; the only requirement is
// that the derived class should provide an appropriate overriding AsString
// method. 
//
// Each instance of TreeViewModel can have multiple client TreeView
// instances, so that the same TreeViewModel instance can be viewed by all
// the clients. The derived TreeViewModel class overrides all those methods
// in its parent class that modify the tree in some way. The overriding
// methods serve as `hooks' to notify the client views of the modification.

#ifdef __GNUC__
#pragma interface
#endif

#include "ui/uidefs.h"

#include "base/tree.h"
#include "base/treewalk.h"
#include "base/objset.h"


class YACL_UI UI_TreeView;

class YACL_UI UI_TreeViewModel: public CL_ObjectPtrTree {

public:
    UI_TreeViewModel ();

    virtual void DestroyContents (CL_TreeNodeLabel l);
    // This is a convenience method to destroy the contents of all the
    // nodes in the subtree rooted at l. THIS METHOD IS NOT YET IMPLEMENTED.
    
    CL_ObjectPtrTreeNode* AddChild (CL_TreeNodeLabel obj,
                                    CL_TreeNodeLabel parent,
                                    long i = 200000L);
    // Override the inherited method.
    
    CL_ObjectPtrTreeNode* NewRoot (CL_TreeNodeLabel l);
    // Override the inherited method.
    
    CL_ObjectPtrTree* ExtractSubtree (CL_TreeNodeLabel obj);
    // Override the inherited method.
    
    bool DestroySubtree (CL_TreeNodeLabel x);
    // Override the inherited method. Caveat: This method refuses to
    // delete the root of the tree; in other words, if called with the
    // root's label, the method simply returns FALSE.

    const char* ClassName () const {return "UI_TreeViewModel";};
    
protected:
    void NodeContentChanged (CL_ObjectPtrTreeNode* node);
    // Override the inherited method. The implementation of this method
    // updates
    
    CL_ObjectSet _clientSet;



public:
    void AddClient (UI_TreeView* client);
    // [For YACL internal use only] Add a client to this model.

    void RemoveClient (UI_TreeView* client);
    // [For YACL internal use only] Remove a client from this model.
    
};

#endif /* _treevmdl_h_ */
