


#include "ui/treeview.h"
#include "ui/font.h"
#include "ui/stddlg.h"


#include "appwin.h"



enum {
    ID_TREEVIEW = 101,
    ID_ADDBTN,
    ID_DELBTN,
    ID_EXPANDBTN,
    ID_COLLAPSEBTN,
    ID_QUERYBTN,
    ID_TEXTEDITOR
};

class NodeLabel: public CL_Object {

public:
    NodeLabel (const char* nodeText) : _nodeText (nodeText) {};

    ~NodeLabel () {};
    
    CL_String AsString () const {return _nodeText;};

protected:
    CL_String _nodeText;

};


static void SetupTree (UI_TreeViewModel& aTree)
{
    NodeLabel* nodeLabel = new NodeLabel ("Id 1 Level 0 (root)");
    aTree.NewRoot (1)->SetContent(nodeLabel);

    nodeLabel = new NodeLabel ("Id 2 Level 1, #1");
    aTree.AddChild (2, 1)->SetContent(nodeLabel);

    nodeLabel = new NodeLabel ("Id 3 Level 1, #2");
    aTree.AddChild (3, 1)->SetContent(nodeLabel); 

    nodeLabel = new NodeLabel ("Id 4 Level 1, #3");
    aTree.AddChild (4, 1)->SetContent(nodeLabel); 

    nodeLabel = new NodeLabel ("Id 5 Level 2, #1");
    aTree.AddChild (5, 3)->SetContent(nodeLabel); 

    nodeLabel = new NodeLabel ("Id 6 Level 2, #2");
    aTree.AddChild (6, 3)->SetContent(nodeLabel); 
}


UI_ViewDescriptor desc [] = {
{View_PushButton, ID_ADDBTN,       520,  50, 150,  30, FALSE, "Add child"},
{View_PushButton, ID_DELBTN,       520, 100, 150,  30, FALSE, "Destroy subtree"},
{View_PushButton, ID_EXPANDBTN   , 520, 150, 150,  30, FALSE, "Expand selection"},
{View_PushButton, ID_COLLAPSEBTN,  520, 200, 150,  30, FALSE, "Collapse selection"},
{View_PushButton, ID_QUERYBTN,     520, 250, 150,  30, FALSE, "Show state"},
{View_TextView,   ID_TEXTEDITOR,    10, 280, 480, 210, FALSE, ""},
{View_None, 0, 0}
};

AppWindow::AppWindow ()
: UI_CompositeVObject (NULL, desc, FALSE, UI_Rectangle (50, 50, 700, 500))
{
    Title() = "YACL TreeView test";
    SetupTree (_aTree);

    _tv = new UI_TreeView (this, UI_Rectangle (10, 10, 480, 250),
                           ID_TREEVIEW, &_aTree);
    _tv->ShowBorder ();
    _tv->SetSeparation (0);
    _tv->SetIndentDepth (60);

    _addChildBtn   = (UI_PushButton*) (*this)[ID_ADDBTN];
    _delSubtreeBtn = (UI_PushButton*) (*this)[ID_DELBTN];
    _expandBtn     = (UI_PushButton*) (*this)[ID_EXPANDBTN];
    _collapseBtn   = (UI_PushButton*) (*this)[ID_COLLAPSEBTN];
    _queryBtn      = (UI_PushButton*) (*this)[ID_QUERYBTN];
    _msg           = (UI_TextEditor*) (*this)[ID_TEXTEDITOR];
    _msg->Font()   = UI_FontDesc (UIFont_Courier, 9);
}

bool AppWindow::HandleChildEvent (const UI_Event& e)
{
    UI_EventType type = e.Type();
    UI_VisualObject* origin = e.Origin();
    if (type == Event_Select && origin == _addChildBtn) {
        long nodeId = _tv->Selection();
        if (nodeId > 0) {
            static long label = 500;
            CL_ObjectPtrTreeNode* node = _aTree.AddChild (label++, nodeId);
            node->SetContent(new NodeLabel ("New child"));
        }
    }
    else if (type == Event_Select && origin == _delSubtreeBtn) {
        long nodeId = _tv->Selection();
        if (nodeId > 0) {
            CL_ObjectPtrTreePostWalker walker (_aTree.Node (nodeId));
            while (walker.More())
                delete walker.Next()->Content();
            _aTree.DestroySubtree (nodeId);
        }
    }
    else if (type == Event_Select && origin == _expandBtn) {
        long nodeId = _tv->Selection();
        if (nodeId > 0)
            _tv->ExpandNode (nodeId);
    }
    else if (type == Event_Select && origin == _collapseBtn) {
        long nodeId = _tv->Selection();
        if (nodeId > 0)
            _tv->CollapseNode (nodeId);
    }
    else if (type == Event_Select && origin == _queryBtn) {
        long nodeId = _tv->Selection();
        if (nodeId > 0) {
            const char* state = _tv->IsExpanded (nodeId) ? "expanded"
                : "collapsed";
            CL_String msg ("The selected node is currently ");
            UI_StandardDialog ((msg + state).AsPtr(), "Info", this);
        }
        else
            UI_StandardDialog ("No current selection", "Info", this);
    }
    else if (origin == _tv && (type == Event_TreeViewNodeCollapsed ||
                             type == Event_TreeViewNodeExpanded ||
                             type == Event_Select)) {
        long nodeId = ((const UI_TreeViewEvent&) e).NodeId ();
        CL_String& model = (CL_String&) _msg->Model();
        model += e.AsString() + " Node id " + CL_String (nodeId) + "\n";
    }
    return TRUE;
}

AppWindow::~AppWindow ()
{
    _aTree.DestroyContents(_aTree.RootLabel());
}
